@@ -16,6 +16,9 @@ var cnames7 = []string{
 	"EXTREG",
 	"SPR",
 	"COND",
+	"ARNG",
+	"ELEM",
+	"LIST",
 	"ZCON",
 	"ABCON0",
 	"ADDCON0",
@@ -35,6 +38,7 @@ var cnames7 = []string{
 	"LBRA",
 	"NPAUTO",
 	"NSAUTO",
+	"PSAUTO_8",
 	"PSAUTO",
 	"PPAUTO",
 	"UAUTO4K_8",
@@ -57,6 +61,7 @@ var cnames7 = []string{
 	"ZOREG",
 	"NPOREG",
 	"NSOREG",
+	"PSOREG_8",
 	"PSOREG",
 	"PPOREG",
 	"UOREG4K_8",
